###
### FUNCTIONS, PACKAGES and GLOBAL VARIABLES for GAP costruction 
###

#########################################################################
##################
################## Packages
##################
#########################################################################

local({pkg <- "boot"; if(nchar(pkg)) library(pkg, character.only=TRUE)})
local({pkg <- "MASS"; if(nchar(pkg)) library(pkg, character.only=TRUE)})
local({pkg <- "mixtools"; if(nchar(pkg)) library(pkg, character.only=TRUE)})
local({pkg <- "diptest"; if(nchar(pkg)) library(pkg, character.only=TRUE)})
local({pkg <- "DNAcopy"; if(nchar(pkg)) library(pkg, character.only=TRUE)})
library(DNAcopy)


#########################################################################
##################
################## Global variables
##################
#########################################################################

### Maximal copy number
maxCN<-8

####### Columns of the main table with segmented profiles

#"Index" "IndxSnpStart" "IndxSnpEnd" "IndxCnStart" "IndxCnEnd" "Chr" "LenSnp" "CN_BP" "AD_BP" "CN" "AllDiff_L" "AllDiff_H"

####  "Index"		index of the segment
####  "IndxSnpStart"    segment starting SNP number in ordered by Chromosome and Position Affy 6.0 chip SNP annotation
####  "IndxSnpEnd"      segment ending SNP number in ordered by Chromosome and Position  Affy 6.0 chip SNP annotation
####  "IndxCnStart"   	segment starting CN number in ordered by Chromosome and Position  Affy 6.0 chip SNP annotation
####  "IndxCnEnd"	segment ending CN number in ordered by Chromosome and Position  Affy 6.0 chip SNP annotation
####  "Chr"             Chromosome number, integer for p arm, integer+0.5 for q arm,
####				X: 23, Y: 24, XY: 25
####  "LenSNP"		Length of a segment in SNP counts
####  "CN_BP"		Break point in CN profile
####  "AD_BP"		Break point in AD profile
####  "CN",             Median of LRR in the segment
####  "AllDiff_L",	One (any) of two AD modes in the segment, 
####  "AllDiff_sm"	Updated Allelic difference for FINAL PREDICTION 
####  "nCN","nBA","nScn" Subsidiary columns, used for recognition,
####  			"nScn" contains smoothed CN signal according to FINAL PREDICTION
####  "Copy","Mallele"  FINAL PREDICTION OF COPY NUMBER and MAJOR ALLELE
####  "Segment_N" 	Segment number according to FINAL PREDICTION
####   "Subclone" 	subclone information (may be absent in the last version)
####  "CN1","BA1","Rank1" First choice prediction
####  "CN2","BA2","Rank2" Second choice prediction
####  "CN3","BA3","Rank3" Third choice prediction

power_Affy<-3/4
power_Cyto<-0.7


# "Index","IndxSnpStart","IndxSnpEnd","IndxCnStart","IndxCnEnd",
c_ind<-1; c_is<-2; c_if<-3; c_isCN<-4; c_ifCN<-5

# "LenSnp","Chr",
c_chr<-6;c_len<-7; 

# "CN_BP" and "AD_BP"  
c_cnBP<-8; c_adBP<-9;

# "CN","AllDiff_L","AllDiff_sm",
c_cn<-10; c_adL<-11; c_adH<-12

# "nCN","nBA","nScn"
c_nC<-13; c_nA<-14; c_nS<-15

# "Copy","Mallele","Segment_N","Subclone",
c_CN<-16; c_BA<-17; c_conf<-18; c_sb<-19

# "CN1","BA1","Rank1","CN2","BA2","Rank2","CN3","BA3","Rank3" 
c_CN1<-20; c_BA1<-21; c_rnk1<-22


ColNamesTMP<-c("Index", "IndxSnpStart", "IndxSnpEnd", "IndxCnStart", "IndxCnEnd",
 "Chr", "LenSnp", "CN_BP", "AD_BP", "CN", "AllDiff_L", "AllDiff_sm",
"nCN","nAD","nScn","Copy","Mallele","Segment_N","Subclone",
"CN1","BA1","Rank1","CN2","BA2","Rank2","CN3","BA3","Rank3")


# pattern of filename with Log ratio vector 
vect_LR_pattern="V_LRR_"

# pattern of filename with Allelic Differences vector 
vect_AD_pattern="V_AD_"

# pattern of filename with Copy number (CN probes in Affy SNP6) vector 
vect_CN_pattern="V_CN_"

# pattern of filename for resulting segmented summary file 
vect_Segm_pattern="segm_"

#############################################################################
#######
################ Constructing GAP: Break points and homo regions detected
#######
#######   Function utilizes circular binary segmentation algorithm (DNAcopy R package)
#######   to find break points in Log ratio and Allelic difference profiles
#######
#######   In order to eliminate on mode from 2 mode Allelic difference distribution
#######   We are superimposing LR and AD profiles and deleting AD points which are in proximity
#######   to some LR point
#######
#######
#############################################################################


#########################################################################
##################
################## Function LogLike
##################
#########################################################################


logLike<- function(x, Lambda, MU, SD){
    a1 <- (x - MU[1])^2
    if(MU[1]!=0){b1 <- (Lambda/SD[1]) * exp(-a1/2/SD[1]^2)
	}else{b1 <- 2*(Lambda/SD[1]) * exp(-a1/2/SD[1]^2)}
    a2 <- (x - MU[2])^2
    b2 <- ((1 - Lambda)/SD[2]) * exp(-a2/2/SD[2]^2)
    out <- sum(log(b1 + b2))-length(x) * 0.918938533204673
    out
}




#########################################################################
##################
################## Function cut tails of Alleic dif distr
##################
#########################################################################

cutADV<-function(adv,prRight,prLeft,centerBoundary){

tt<-which(is.na(adv))
if((length(tt)!=0)){adv<-adv[-tt]}

if(length(adv)==0){adv<-numeric(0)}else{
tt<-which(adv>quantile(adv, prob=prRight))
if(length(tt)>0){adv<-adv[-tt]}

tt<-length(which(adv<centerBoundary))/length(adv)

if(tt<prLeft){
	tt<-which(adv<quantile(adv, prob=prLeft))
	if(length(tt)>0){adv<-adv[-tt]}
}
}
out<-adv

}
#########################################################################
##################
################## Function Mixture of Two Normals
##################
#########################################################################

normalmixEM2compModif<-function (x, lambda, mu, sigsqrd, eps = 1e-08, maxit = 1000) 
{
    mi1<-c(mu[1],sigsqrd[1],lambda);names(mi1)<-c("mu","D","la")
    mi2<-c(mu[2],sigsqrd[2],1-lambda);names(mi2)<-c("mu","D","la")

    mx <- mean(x)
    const <- length(x) * 0.918938533204673
    dl <- 1 + eps
    iter <- 0
    a1 <- (x - mi1[1])^2
    b1 <- (mi1[3]/sqrt(mi1[2])) * exp(-a1/2/mi1[2])
    a2 <- (x - mi2[1])^2
    b2 <- (mi2[3]/sqrt(mi2[2])) * exp(-a2/2/mi2[2])
    l<-sum(log(b1 + b2))
    ll <- c(mi1,mi2,l)
    postprobs <- b1/(b1 + b2)
    mi1[3] <- mean(postprobs)
    mi2[3]<- (1 - mi1[3])

    while (!is.na(dl) & dl > eps && iter < maxit && mi1[3]*mi2[3]>0) {

       iter <- iter + 1
        mi1[1] <- mean(postprobs * x)/mi1[3]
        mi2[1] <- (mx - mi1[3] * mi1[1])/mi2[3]
        mi1[2] <- mean(postprobs * a1)/mi1[3]
        mi2[2] <- mean((1 - postprobs) * a2)/mi2[3]

        a1 <- (x - mi1[1])^2
        b1 <- (mi1[3]/sqrt(mi1[2])) * exp(-a1/2/mi1[2])
        a2 <- (x - mi2[1])^2
        b2 <- (mi2[3]/sqrt(mi2[2])) * exp(-a2/2/mi2[2])

        oldl <- l
        l <- sum(log(b1 + b2))
		
        dl <- l - oldl
        ll <- cbind(ll,c(mi1,mi2,l))

        postprobs <- b1/(b1 + b2)
        mi1[3] <- mean(postprobs)
	  mi2[3]<- (1 - mi1[3])
    }

  if(is.na(dl)){ll<-c(NA,mi1[2],mi2[2])

  }else{
  if (mi1[3]==0){ mi2[1] <- mx;mi2[2] <- mean((x - mi2[1])^2)
			l<-logLike(x, Lambda=0, MU=c(mi1[1],mi2[1]), SD=c(sqrt(mi1[2]),sqrt(mi2[2])))
  	      	ll <- cbind(ll,c(mi1,mi2,l+const))
	
  }else{if (mi2[3]==0){ mi1[1] <- mx;mi1[2] <- mean((x - mi1[1])^2)
			l<-logLike(x, Lambda=1, MU=c(mi1[1],mi2[1]), SD=c(sqrt(mi1[2]),sqrt(mi2[2])))
  	      	ll <- cbind(ll,c(mi1,mi2,l+const))
	
		  }
		}

  ll[2,]<-sqrt(ll[2,])
  ll[5,]<-sqrt(ll[5,])
  ll[7,]<-ll[7,]-const 
}
  out <- ll
  out

}





#########################################################################
##################
################## Function Choose Two Normals Mixture Model
##################
#########################################################################


chooseMix2Model_new<-function(x, lambda, mu, sigsqrd, eps, maxit){

ll<-c(NA,1,1)
while(is.na(ll[1])){
	if(is.na(ll[1])){if(ll[2]==0){x<-x[-which.min(x)]}else{if(ll[3]==0){x<-x[-which.max(x)]}}}
	ll<-normalmixEM2compModif(x, lambda, mu, sigsqrd, eps, maxit) 
}

ll<-ll[,order(ll[7,],decreasing=T)]


if(ll[1,1]<0.35){

	mu2<-ll[4,1]
	sd2<-ll[5,1]
	sd1<-ll[2,1]

	mu<-c(0,0.15,0.2,0.25,0.3)
	lambda<-c(1,0.2,0.4,0.6)
	
	iter<-length(mu)*length(lambda)
	ll<-cbind(matrix(0,7,iter),ll)
	ll[2,1:iter]<-sd1
	ll[4,1:iter]<-mu2
	ll[5,1:iter]<-sd2

	k<-1;i<-1
	for(k in 1:length(mu)){
	for(i in 1:length(lambda)){
		iter<-(k-1)*length(lambda)+i
		ll[1,iter]<-mu[k]
		ll[3,iter]<-lambda[i]
		ll[6,iter]<-1-lambda[i]
		ll[7,iter]<-logLike(x, Lambda=lambda[i], MU=c(mu[k],mu2), SD=c(sd1,sd2))
#normalmixEM2compSDfit(x, lambda=lambda[i], mu=c(mu[k],mu2), sigsqrd=c(sd1,sd2), eps = 1e-01, maxit = 20) 
	}#i
	}#k

ll<-ll[,order(ll[7,],decreasing=T)]

#if close to balance
}else{

}

out<-ll[,1]


}# function



########################################################################
##################
################## Function SD of homoAD band
##################
#########################################################################



getSDofHomozygousADband<-function(tmp, datL, Length){

sdH<-numeric(0)

datL<-exp(log(2)*datL)

k<-4
for(k in 1:dim(tmp)[[1]]){
	if(tmp[k,c_len]>Length){
		adv<-datL[tmp[k,c_is]:tmp[k,c_if]]
		adv<-cutADV(adv,prRight=0.97,prLeft=0.03,centerBoundary=0)
		sdH<-c(sdH,adv-median(adv))
	}
}#k

if(length(sdH)>0){sdH<-sd(sdH,na.rm=T)}else{sdH<-sd(datL,na.rm=T)}

out<-sdH

}



#########################################################################
##################
################## Function AllelicDiff for recognition
##################
#########################################################################

getADforGAPrecogn<-function(tmp,datA,sdH){

k<-480
for(k in 1:dim(tmp)[[1]]){

adv<-datA[tmp[k,c_is]:tmp[k,c_if]]
adv<-cutADV(adv,prRight=0.85,prLeft=0.05,centerBoundary=0.2)

if(length(adv)>15){
	
	hh<-hist(adv, breaks=seq(0,max(1,round(max(adv)+1)),0.1),right=F,plot=F)

	if(length(adv)>50){
	
		dd<-density(adv,n=20)
		
		MaxXY<-numeric(0)
		Id<-0

		for(s in 2:length(dd$x)){
			if((dd$y[s]<dd$y[s-1])&&(Id==0)){
				MaxXY<-c(MaxXY,s-1);Id<-1
				}else{if((dd$y[s]>dd$y[s-1])&&(Id==1)){Id<-0}}
		}#s

		if(sum(hh$counts[1:2])/sum(hh$counts)>0.2){
			if((hh$counts[1]>hh$counts[2]*1.2)&&(hh$counts[1]>hh$counts[3]*1.3)){
				tmp[k,c_adL]<-0
			}else{
				if((hh$counts[2]>hh$counts[1]*1.2)&&(hh$counts[2]>hh$counts[3]*1.2)){
					if(dd$x[MaxXY[1]]<0.3){tmp[k,c_adL]<-dd$x[MaxXY[1]]
					}else{tmp[k,c_adL]<-hh$mids[2]}
				}else{
					ll<-chooseMix2Model_new(x=adv, lambda=0.4, mu=c(quantile(adv,prob=0.2),quantile(adv,prob=0.8)),
						 sigsqrd=c(sdH^2,sdH^2), eps = 1e-02, maxit = 200)
					if(ll[3]>=0.2){tmp[k,c_adL]<-ll[1]
					}else{tmp[k,c_adL]<-dd$x[which.max(dd$y)]}	
					}
			}
		}else{
			
			if((length(MaxXY)==2)&&(MaxXY[2]>MaxXY[1]+4)
				&&(min(dd$y[(MaxXY[1]+2):(MaxXY[2]-2)])<0.8*min(dd$y[MaxXY[1]],dd$y[MaxXY[2]]))){

					if(length(adv)>1000){
						if(dd$x[MaxXY[1]]>=0.2){tmp[k,c_adL]<-dd$x[MaxXY[1]]
						}else{
							ll<-chooseMix2Model_new(x=adv, lambda=0.4, mu=c(quantile(adv,prob=0.2),quantile(adv,prob=0.8)),
								 sigsqrd=c(sdH^2,sdH^2), eps = 1e-02, maxit = 200)
							if((ll[3]>=0.2)&(ll[1]>=0.18)){tmp[k,c_adL]<-ll[1]
							}else{tmp[k,c_adL]<-dd$x[MaxXY[2]]}

							}

					}else{	if(dd$x[MaxXY[1]]<0.3){	tmp[k,c_adL]<-dd$x[MaxXY[2]]
						}else{tmp[k,c_adL]<-dd$x[MaxXY[1]]}
						}				
			}else{
				ll<-chooseMix2Model_new(x=adv, lambda=0.4, mu=c(quantile(adv,prob=0.2),quantile(adv,prob=0.8)),
					 sigsqrd=c(sdH^2,sdH^2), eps = 1e-02, maxit = 200)
				if(ll[3]>=0.2){tmp[k,c_adL]<-ll[1]
				}else{tmp[k,c_adL]<-dd$x[which.max(dd$y)]}	
				}
			}
	}else{tmp[k,c_adL]<-hh$mids[which.max(hh$counts)]}

}else{

if(length(adv)>0){
	hh<-hist(adv, breaks=seq(0,max(1,round(max(adv)+1)),0.2),plot=F)
	tmp[k,c_adL]<-hh$mids[which.max(hh$counts)]
}else{tmp[k,c_adL]<-NA}

}

}#k


tmp<-round(tmp,4)
out<-tmp

}#function




################################################################
####
####
####  Function producing one band distribution of Allelic Difference vector
####
####
################################################################

getADbandforSegmentation<-function(datA,datL,dataF,c_bpC){


##### 2. estimating sd of LRR supposed to be equal to sd of homoAD band
Length<-500
sdH<-numeric(0)
nSNP<-dim(SNP_annot)[[1]]

datL<-exp(log(2)*datL)

tt<-which(abs(SNP_annot[2:nSNP,C_Chr]-SNP_annot[1:(nSNP-1),C_Chr])+abs(SNP_annot[2:nSNP,C_Chr+1]-SNP_annot[1:(nSNP-1),C_Chr+1])!=0)
tt<-c(1,tt,tt+1)
tt<-sort(union(tt,which(dataF[,c_bpC]==1)))

if(length(tt)>1){
for(k in 2:length(tt)){
	if(tt[k]-tt[k-1]>Length){
		adv<-datL[tt[k-1]:tt[k]]
		adv<-cutADV(adv,prRight=0.97,prLeft=0.03,centerBoundary=0)
		sdH<-c(sdH,adv-median(adv))
	}
}#k
}
if(length(sdH)>0){sdH<-sd(sdH,na.rm=T)}else{sdH<-sd(datL,na.rm=T)}

######  3. get homoConst - median of homozygous AD band

tt<-which(abs(datL-abs(datA))<1.5*sdH)

homoConst<-median(datA[tt])


##########################
####  Processing AD to make one band distribution

#### 1. Deleting AllDiff above smoothed LRR

ad<-datA

#tt<-which(is.na(ad));if(length(tt)>0){ad[tt]<-abs(datL[tt]-1+homoConst);datA[tt]<-ad[tt]}

tt<-which(ad-datL+1-homoConst>0);if(length(tt)>0){ad[tt]<-100}

for(k in 1:5){
	aa<-c(0,0,(datL[(5-k+1):(nSNP-k+1)]-1+homoConst),0,0)
	tt<-which(abs(ad-aa)<1.7*sdH)
	if(length(tt)>0){ad[tt]<-100}
}

tt<-which(ad==100);if(length(tt)>0){ad[tt]<-(-1)}

# ad is a vector with Allelic differences with germline homozygous band set to (-1)


out<-list(ad=ad, sdH=sdH, homoConst=homoConst)


}#function




#############################################################################
#######
#######      MAIN FUNCTION: Breakpoint and Homozygosity regions detection
#######     and Constructing GAP: Uniting breakpoints in CN and AD profiles 
#######           into segmented summary table
#######   
#######
#############################################################################

CBS_breakpoints_and_GAPconstruction<-function(ii){

##### 1.2 Reading data vectors

datA<-read.table(sub(vect_LR_pattern,vect_AD_pattern,segFiles[ii]),header=F)
datA<-data.matrix(datA)
datA<-abs(as.numeric(datA))
ad<-datA
tt<-which(is.na(datA))
if(length(tt)>0){datA[tt]<-0}


datC<-read.table(sub(vect_LR_pattern,vect_CN_pattern,segFiles[ii]),header=F)
datC<-data.matrix(datC)
datC<-as.numeric(datC)
tt<-which(is.na(datC))
if(length(tt)>0){datC[tt]<-0}


datL<-read.table(segFiles[ii],header=F)
datL<-data.matrix(datL)
datL<-as.numeric(datL)
tt<-which(is.na(datL))
if(length(tt)>0){datL[tt]<-0}

nSNP<-length(datA)

dataF<-matrix(0,nSNP,4)
c_bpC<-1;c_bpA<-2;c_hetAD<-3;c_homo<-4
colnames(dataF)<-c("BP_CN","BP_AD","AD_hete","Homo_segm")


##################### Segmenting with CBS

##### 1. segment CN

CN<-as.numeric(SNP_annot[,C_Pos+1])
tt<-which(CN!=0)

CNA.object <- CNA(datC[CN[tt]],round((SNP_annot[tt,C_Chr]+(SNP_annot[tt,C_Chr+1]-1)/2),1),SNP_annot[tt,C_Ind],data.type="logratio",sampleid=ii)
CNA.object <- smooth.CNA(CNA.object, smooth.region=2, outlier.SD.scale=2.5, smooth.SD.scale=2,trim=0.1)

segment.CNA.object <- segment(CNA.object,alpha = 0.005, nperm = 10000, p.method = "hybrid", min.width=5, kmax=25, nmin=200, 
       eta=0.05, trim = 0.025, undo.splits = "none", verbose=1)

tmp <- segment.CNA.object$output
rm(segment.CNA.object,CNA.object)

dataF[tmp$loc.end,c_bpC]<-1

rm(tmp)



####### Segment baf

   ###### Get one band AD and two important constanst 
    ######   sdH: sd of AD homozygous band 
    ######   homoConst: parameter characterizing datA-datL (always 1 in Affy SNP6, vary in 0.8-1 in Cytoscan) 

    ADetc<-getADbandforSegmentation(datA,datL,dataF,c_bpC)
    ad<-ADetc$ad
    sdH<-ADetc$sdH
    homoConst<-ADetc$homoConst
    rm(ADetc)


####### Segmenting one band AD 

tmp<-cbind(ad,round((SNP_annot[,C_Chr]+(SNP_annot[,C_Chr+1]-1)/2),1),SNP_annot[,C_Ind])
tmp<-data.matrix(tmp)

tt<-which(ad<0);if(length(tt)>0){tmp<-tmp[-tt,]}

dataF[,c_hetAD]<-1

dataF[tt,c_hetAD]<-0

CNA.object <- CNA(tmp[,1],tmp[,2],tmp[,3],data.type="logratio",sampleid=ii)
CNA.object <- smooth.CNA(CNA.object, smooth.region=4, outlier.SD.scale=4, smooth.SD.scale=1,trim=0.05)

segment.CNA.object <- segment(CNA.object,alpha = 0.0005, nperm = 10000, p.method = "hybrid", min.width=5, kmax=25, nmin=200, 
       eta=0.05, trim = 0.025, undo.splits = "none", verbose=1)

tmp <- segment.CNA.object$output
rm(segment.CNA.object,CNA.object)

dataF[tmp$loc.end,c_bpA]<-1
rm(tmp)


###########################
#### 2. Marking regions of heterozygosity
####
#### SNP belongs to heterozygous region if there are:
####   1) <23 homozygous SNPs in 31 SNP window 
####   2) <11 homozygous SNPs from each side of SNP
####  >180 homozygous SNPs in 200 SNPs window determine homozygous region
####

hAD<-1-sign(ad+1)

rh<-hAD*0
lh<-hAD*0
for(k in 1:15){
	lh[16:length(lh)]<-lh[16:length(lh)]+hAD[(15-k+1):(length(hAD)-k)]
	rh[1:(length(rh)-16+1)]<-rh[1:(length(rh)-16+1)]+hAD[(k+1):(length(hAD)-15+k)]
}

S_hete<-hAD*0
tt<-which(rh+lh<23);if(length(tt)>0){S_hete[tt]<-1}
tt<-which(rh<11);if(length(tt)>0){S_hete[tt]<-1}
tt<-which(lh<11);if(length(tt)>0){S_hete[tt]<-1}

S_homo<-(1-S_hete)

rh<-S_homo*0
lh<-S_homo*0
for(k in 1:99){
	lh[100:length(lh)]<-lh[100:length(lh)]+S_homo[(99-k+1):(length(S_homo)-k)]
	rh[1:(length(rh)-100+1)]<-rh[1:(length(rh)-100+1)]+S_homo[(k+1):(length(S_homo)-99+k)]
}

S_homo<-S_homo*0
tt<-which(rh+lh>180);if(length(tt)>0){S_homo[tt]<-1}
tt<-which(rh>90);if(length(tt)>0){S_homo[tt]<-1}
tt<-which(lh>90);if(length(tt)>0){S_homo[tt]<-1}

S_homo<-S_homo*(1-S_hete)

seg<-abs(S_homo[2:nSNP]-S_homo[1:(nSNP-1)])+
abs(dataF[2:nSNP,3]-dataF[1:(nSNP-1),3])+abs(dataF[2:nSNP,2]-dataF[1:(nSNP-1),2])

seg<-c(1,which(seg!=0),nSNP)
seg<-as.numeric(rownames(table(seg)))

for(k in 2:length(seg)){if(seg[k]-seg[k-1]+1<200){S_homo[seg[k-1]:seg[k]]<-0}}

dataF[,c_homo]<-S_homo

###########################################
###################  Segmented summary file construction

nSNP<-dim(dataF)[[1]]

c_bpC<-1;c_bpA<-2;c_hetAD<-3;c_homo<-4

seg<-abs(SNP_annot[1:(nSNP-1),C_Chr]-SNP_annot[2:(nSNP-0),C_Chr])+
	abs(SNP_annot[1:(nSNP-1),C_Chr+1]-SNP_annot[2:(nSNP-0),C_Chr+1])+
	dataF[1:(nSNP-1),c_bpC]+dataF[1:(nSNP-1),c_bpA]+
	abs(dataF[1:(nSNP-1),c_homo]-dataF[2:(nSNP-0),c_homo])

seg<-which(seg!=0)
seg<-c(seg,nSNP)
seg<-cbind(data.matrix(SNP_annot[seg,]),dataF[seg,])

tmp<-matrix(0,dim(seg)[[1]],12)

colnames(tmp)<-ColNamesTMP[1:12]

tmp[,1]<-seq(1,dim(tmp)[[1]])

tmp[,c_cnBP]<-seg[,c_bpC+5]
tmp[,c_adBP]<-seg[,c_bpA+5]
tmp[,c_adH]<-seg[,c_homo+5]


tmp[,c_chr]<-seg[,C_Chr]+(seg[,C_Chr+1]-1)/2

tmp[,c_if]<-seg[,C_Ind]
tmp[,c_is]<-c(1,seg[1:(dim(seg)[[1]]-1),C_Ind]+1)

tmp[,c_len]<-tmp[,c_if]-tmp[,c_is]+1


################### Calculating Breaks coordinates for CN profile
NL<-nSNP

for(k in 1:dim(tmp)[[1]]){
tt<-tmp[k,c_is]+which(SNP_annot[tmp[k,c_is]:tmp[k,c_if],5]>0)-1
if(length(tt>0)){
tmp[k,c_isCN]<-min(SNP_annot[tt,5])
tmp[k,c_ifCN]<-max(SNP_annot[tt,5])
}
}


Flag<-T
while(Flag){
Flag<-F

tt<-which(tmp[,c_isCN]==0)
for(k in 2:length(tt)){
	if(tt[k-1]+1==tt[k]){
	if(tmp[tt[k-1],c_chr]==tmp[tt[k],c_chr]){
		tmp[tt[k],c_is]<-tmp[tt[k-1],c_is]
		tmp[tt[k],c_len]<-tmp[tt[k-1],c_len]+tmp[tt[k],c_len]
		tmp[tt[k-1],c_len]<-0
	}
	}
}

tt<-which(tmp[,c_len]==0)
if(length(tt)!=0){tmp<-tmp[-tt,];Flag<-T}

}#Flag



#################################################
############### Adjusting between chromosome and arm connections
######### Min segment at chromosome or arm ends should be more than 15 SNPs 

Flag<-T
while(Flag){
Flag<-F
for(k in 2:(dim(tmp)[[1]]-2)){
	if(tmp[k,c_chr]!=tmp[k+1,c_chr]){
		if(((tmp[k,c_len]<15)||(tmp[k,c_isCN]==0))&&(tmp[k,c_chr]==tmp[k-1,c_chr])){
			tmp[k-1,c_if]<-tmp[k,c_if]
			tmp[k-1,c_len]<-tmp[k-1,c_len]+tmp[k,c_len]
			tmp[k-1,c_ifCN]<-max(tmp[k,c_ifCN],tmp[k-1,c_ifCN])
			tmp[k,c_len]<-0
		}
		if(((tmp[k+1,c_len]<15)||(tmp[k+1,c_isCN]==0))&&(tmp[k+1,c_chr]==tmp[k+2,c_chr])){
			tmp[k+2,c_is]<-tmp[k+1,c_is]
			tmp[k+2,c_isCN]<-max(tmp[k+1,c_isCN],tmp[k+2,c_isCN])
			tmp[k+2,c_len]<-tmp[k+1,c_len]+tmp[k+2,c_len]
			tmp[k+1,c_len]<-0
		}
	}
}#k

if(((tmp[1,c_len]<15)||(tmp[1,c_isCN]==0))&&(tmp[1,c_chr]==tmp[2,c_chr])){
	tmp[2,c_is]<-tmp[1,c_is]
	tmp[2,c_len]<-tmp[1,c_len]+tmp[2,c_len]
	tmp[2,c_isCN]<-max(tmp[1,c_isCN],tmp[2,c_isCN])
	tmp[1,c_len]<-0
}

tt<-dim(tmp)[[1]]
if(((tmp[tt,c_len]<15)||(tmp[tt,c_isCN]==0))&&(tmp[tt,c_chr]==tmp[tt-1,c_chr])){
	tmp[tt-1,c_if]<-tmp[tt,c_if]
	tmp[tt-1,c_ifCN]<-max(tmp[tt,c_ifCN],tmp[tt-1,c_ifCN])
	tmp[tt-1,c_len]<-tmp[tt-1,c_len]+tmp[tt,c_len]
	tmp[tt,c_len]<-0
}

tt<-which(tmp[,c_len]==0)
if(length(tt)!=0){tmp<-tmp[-tt,];Flag<-T}

}#Flag

################### Adjusting small regions with no CN points
### Segments with only SNP probes (without CN probes) are linked to adjuscent segments


tt<-which(tmp[,c_isCN]==0)

if(length(tt)>0){
k<-3
for(k in 1:length(tt)){
	if((tt[k]<dim(tmp)[[1]])&&(tt[k]>1)&&(tmp[tt[k]-1,c_chr]==tmp[tt[k]+1,c_chr])){
		if(tmp[tt[k]-1,c_ifCN]+1<tmp[tt[k]+1,c_isCN]){
			tmp[tt[k],c_isCN]<-tmp[tt[k]-1,c_ifCN]+1
			tmp[tt[k],c_ifCN]<-tmp[tt[k]+1,c_isCN]-1
		}else{if((tmp[tt[k],c_adBP]==1)&&(tmp[tt[k]+1,c_adBP]==0)){
				tmp[tt[k]+1,c_is]<-tmp[tt[k],c_is]
				tmp[tt[k]+1,c_len]<-tmp[tt[k],c_len]+tmp[tt[k]+1,c_len]
				tmp[tt[k],c_len]<-0
			}else{kk<-trunc(tmp[tt[k],c_len]/2)
				tmp[tt[k]-1,c_if]<-tmp[tt[k]-1,c_if]+kk
				tmp[tt[k]-1,c_len]<-tmp[tt[k]-1,c_len]+kk

				kk<-tmp[tt[k],c_len]-kk
				tmp[tt[k]+1,c_is]<-tmp[tt[k]+1,c_is]-kk
				tmp[tt[k]+1,c_len]<-tmp[tt[k]+1,c_len]+kk
				tmp[tt[k],c_len]<-0
				}
			}
	}else{print("ZERO!!!")}
}


tt<-which(tmp[,c_len]==0)
if(length(tt)!=0){tmp<-tmp[-tt,]}
}


#print(table(tmp[2:dim(tmp)[[1]],c_is]-tmp[1:(dim(tmp)[[1]]-1),c_if]))

tmp[,c_len]<-tmp[,c_if]-tmp[,c_is]+1

if(length(which(tmp[,c_len]<=0))!=0){print(fileNames[ii]);print("kosyak")}

tt<-which(tmp[,c_chr]==21)
if(length(tt)>0){
	for(i in 1:length(tt)){
		tmp[tt[i]+1,c_is]<-tmp[tt[i],c_is]
		tmp[tt[i]+1,c_isCN]<-tmp[tt[i],c_isCN]
		tmp[tt[i]+1,c_len]<-tmp[tt[i],c_len]+tmp[tt[i]+1,c_len]
		tmp[tt[i],c_ind]<-0
	}
tt<-which(tmp[,c_ind]==0);if(length(tt)!=0){tmp<-tmp[-tt,]}
}

tmp[,c_ind]<-seq(1,dim(tmp)[[1]])

#################################################"
############ Calculating CN and AD modes

########################

datA<-read.table(sub(vect_LR_pattern,vect_AD_pattern,segFiles[ii]),header=F)
datA<-data.matrix(datA)
datA<-abs(as.numeric(datA))

tmp<-getADforGAPrecogn(tmp,datA=abs(datA),sdH)


for(k in 1:dim(tmp)[[1]]){
	if(tmp[k,c_ifCN]>tmp[k,c_isCN]){
	tmp[k,c_cn]<-median(datC[tmp[k,c_isCN]:tmp[k,c_ifCN]],na.rm=T)
	}else{tmp[k,c_cn]<-datC[tmp[k,c_isCN]]}	
}#k


tmp<-round(tmp,4)

#colnames(tmp)[12]<-paste(colnames(tmp)[12],"_homoConst",as.character(round(homoConst,2)),"_sdH",as.character(round(sdH,2)),sep="")

out<-list(tmp=tmp,homoConst=homoConst,sdH=sdH)

}#function



#############################################################################
#######
#######		 GAP Pattern recognition function saving GAP pattern into 
#######            jpeg format
#######
#############################################################################

GAP_pattern<-function(seg_BAF,seg_LRR,seg_len,homoConst,sdH,powerG,showGAPTemplate,saveGAPplot,GAPplot_fileName){

if(showGAPTemplate){
	plot(seg_BAF,seg_LRR,cex=seg_len/quantile(seg_len,prob=0.9),main="GAP",
		xlab="Allelic Difference",ylab="Log Ratio")
	points(seg_BAF,seg_LRR,cex=seg_len/quantile(seg_len,prob=0.9))
	abline(-1,1)
	

}


if(saveGAPplot){
jpeg(file=GAPplot_fileName,height=1200,width=1000,pointsize=24)
}

result<-CopyNumber_and_Genotype_pattern(seg_BAF,seg_LRR,seg_len,homoConst,sdH,powerG,showGAPTemplate=T)

if(saveGAPplot){
dev.off()
}

GAP_pattern<-c(as.numeric(result$coeff[1]),as.numeric(result$coeff[2]),as.numeric(result$Centr_CL[5,4]))


}#function

